#include <lauxlib.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/limits.h>

#include "options.h"
#include "model.h"

void init(options_t *opts, lua_State *L) {
    if (opts->conf == default_opts.conf) exit(0);

    luaL_dofile(L , opts->conf);

#define replace_default(field, var, type)\
    if (opts->field == default_opts.field) {\
        lua_getglobal(L, var);\
        opts->field = luaL_check##type(L, -1);\
    }

    replace_default(wdir, "working_directory", string);
    replace_default(mdir, "models_directory", string);
    replace_default(type, "model_type", string);
    replace_default(iter, "iterations_number", int);
    replace_default(step, "dumping_step", int);
}

void run(options_t opts, void *M) {
    model_dump(M, 0);
    for (int i = 1; i < opts.iter; i++) {
        printf("\r%d", i);
        fflush(stdout);

        model_step(M, i);
        if (i % opts.step == 0) model_dump(M, i);
    }
    if (opts.iter > 0) printf("\n");
}

void *model(options_t opts, lua_State *L) {
    char lib_path[PATH_MAX];
    snprintf(lib_path, PATH_MAX, "%s/%s.so", opts.mdir, opts.type);
    void *M = model_new(lib_path, L);
    return M;
}

int main(int argc, char **argv) {
    options_t opts = parse_cl(argc, argv);
    lua_State *L = luaL_newstate();
    luaL_openlibs(L);
    init(&opts, L);

    void *M = model(opts, L);
    chdir(opts.wdir);
    run(opts, M);

    lua_close(L);
    model_del(M);

    return 0;
}
