#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <limits.h>

#include "options.h"

static void print_usage(FILE *fp, const char *prog_name, int exit_code) {
    fprintf(fp, "usage: %s [options]\n", prog_name);
    fprintf(fp, "    -h, --help\n");
    fprintf(fp, "    -c, --conf <path>       config file\n");
    fprintf(fp, "    -w, --wdir <path>       working directory\n");
    fprintf(fp, "    -m, --mdir <path>       directory containing models\n");
    fprintf(fp, "    -t, --type <name>       type of model to be used\n");
    fprintf(fp, "    -i, --iter <integer>    number of iterations\n");
    fprintf(fp, "    -s, --step <integer>    interval between iterations that will be stored\n");
    exit(exit_code);
}

options_t parse_cl(int argc, char **argv) {
    const char* const short_opts= "hc:w:m:t:i:s:";
    const struct option long_opts[] = {
        { "help", 0, NULL, 'h' },
        { "conf", 1, NULL, 'c' },
        { "wdir", 1, NULL, 'w' }, 
        { "mdir", 1, NULL, 'm' }, 
        { "type", 1, NULL, 't' },
        { "iter", 1, NULL, 'i' },
        { "step", 1, NULL, 's' },
        {  NULL , 0, NULL,  0  },
    };
    options_t opts = default_opts;
    int next_opt;
    do {
        next_opt = getopt_long(argc, argv, short_opts, long_opts, NULL);
        switch (next_opt) {
            case 'h':
                print_usage(stdout, argv[0], 0);
            case 'c':
                opts.conf = optarg;
                break;
            case 'w':
                opts.wdir = optarg;
                break;
            case 'm':
                opts.mdir = optarg;
                break;
            case 't':
                opts.type = optarg;
                break;
            case 'i':
                opts.iter = atoi(optarg);
                break;
            case 's':
                opts.step = atoi(optarg);
                break;
            case '?':
                print_usage(stderr, argv[0], 1);
            case -1:
                break;
            default:
                abort();
        }
    } while (next_opt != -1);
    return opts;
}
